@@ -46,7 +46,7 @@ value ocaml_location (fname, lnum, bolp, lnuml, bolpl, bp, ep) =
   END
 ;
 
-IFDEF OCAML_VERSION >= OCAML_4_00 THEN
+IFDEF OCAML_VERSION >= OCAML_3_01_1 THEN
 value loc_none =
   let loc =
     {Lexing.pos_fname = "_none_"; pos_lnum = 1; pos_bol = 0; pos_cnum = -1}
@@ -94,6 +94,35 @@ value list_map_check f l =
     | [] -> Some (List.rev rev_l) ]
 ;
 
+IFDEF OCAML_VERSION >= OCAML_4_03_0 THEN
+  value labelled lab =
+    if lab = "" then Nolabel
+    else if lab.[0] = '?' then
+      Optional (String.sub lab 1 (String.length lab - 1))
+    else Labelled lab;
+END;
+
+IFDEF OCAML_VERSION > OCAML_3_01_1 AND OCAML_VERSION < OCAML_4_03_0 THEN
+  value mkopt t lab =
+    if lab = "" then t
+    else if lab.[0] = '?' then
+      IFDEF OCAML_VERSION < OCAML_3_11_0 THEN
+        {ptyp_desc = Ptyp_constr (mknoloc (Lident "option")) [t];
+         ptyp_loc = loc_none}
+      ELSIFDEF OCAML_VERSION < OCAML_4_02 THEN
+        {ptyp_desc =
+           Ptyp_constr (mknoloc (Ldot (Lident "*predef*") "option")) [t];
+         ptyp_loc = loc_none}
+      ELSE
+        {ptyp_desc =
+           Ptyp_constr (mknoloc (Ldot (Lident "*predef*") "option")) [t];
+         ptyp_loc = loc_none;
+         ptyp_attributes = []}
+      END
+    else t
+  ;
+END;
+
 value ocaml_value_description vn t p =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN {pval_type = t; pval_prim = p}
   ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
@@ -354,8 +383,14 @@ value ocaml_ptype_variant ctl priv =
             (fun (c, tl, rto, loc) ->
                if rto <> None then raise Exit
                else
-                 {pcd_name = mkloc loc c; pcd_args = tl; pcd_res = None;
-                  pcd_loc = loc; pcd_attributes = []})
+                 IFDEF OCAML_VERSION < OCAML_4_03_0 THEN
+                   {pcd_name = mkloc loc c; pcd_args = tl; pcd_res = None;
+                    pcd_loc = loc; pcd_attributes = []}
+                 ELSE
+                   let tl = Pcstr_tuple tl in
+                   {pcd_name = mkloc loc c; pcd_args = tl; pcd_res = None;
+                    pcd_loc = loc; pcd_attributes = []}
+                 END)
             ctl
         in
         Some (Ptype_variant ctl)
@@ -366,8 +401,10 @@ value ocaml_ptype_variant ctl priv =
 ;
 
 value ocaml_ptyp_arrow lab t1 t2 =
-  IFDEF OCAML_VERSION <= OCAML_2_04 THEN Ptyp_arrow t1 t2
-  ELSE Ptyp_arrow lab t1 t2 END
+  IFDEF OCAML_VERSION < OCAML_3_00 THEN Ptyp_arrow t1 t2
+  ELSIFDEF OCAML_VERSION <= OCAML_3_01 THEN Ptyp_arrow lab t1 t2
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN Ptyp_arrow lab (mkopt t1 lab) t2
+  ELSE Ptyp_arrow (labelled lab) t1 t2 END
 ;
 
 value ocaml_ptyp_class li tl ll =
@@ -445,10 +482,43 @@ value ocaml_package_type li ltl =
   (mknoloc li, List.map (fun (li, t) → (mkloc t.ptyp_loc li, t)) ltl)
 ;
 
+value ocaml_pconst_char c =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Const_char c
+  ELSE Pconst_char c END
+;
+value ocaml_pconst_int i =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Const_int i
+  ELSE Pconst_integer (string_of_int i) None END
+;
+value ocaml_pconst_float s =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Const_float s
+  ELSE Pconst_float s None END
+;
+
 value ocaml_const_string s =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Const_string s
   ELSE Const_string s None END
 ;
+value ocaml_pconst_string s so =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Const_string s
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN Const_string s so
+  ELSE Pconst_string s so END
+;
+
+value pconst_of_const =
+  fun
+  [ Const_int i -> ocaml_pconst_int i
+  | Const_char c -> ocaml_pconst_char c
+  | IFDEF OCAML_VERSION < OCAML_4_02 THEN
+      Const_string s -> ocaml_pconst_string s None
+    ELSE
+      Const_string s so -> ocaml_pconst_string s so
+    END
+  | Const_float s -> ocaml_pconst_float s
+  | IFDEF OCAML_VERSION > OCAML_3_01 THEN
+    _ -> failwith "pconstant of constant"
+    END]
+;
 
 value ocaml_const_int32 =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
@@ -467,7 +537,8 @@ value ocaml_const_nativeint =
 
 value ocaml_pexp_apply f lel =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN Pexp_apply f (List.map snd lel)
-  ELSE Pexp_apply f lel END
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN Pexp_apply f lel
+  ELSE Pexp_apply f (List.map (fun (l, e) -> (labelled l, e)) lel) END
 ;
 
 value ocaml_pexp_assertfalse fname loc =
@@ -586,7 +657,9 @@ value ocaml_pexp_function lab eo pel =
   ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pexp_function lab eo pel
   ELSE
     match pel with
-    | [{pc_lhs = p; pc_guard = None; pc_rhs = e}] -> Pexp_fun lab eo p e
+    | [{pc_lhs = p; pc_guard = None; pc_rhs = e}] ->
+        IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Pexp_fun lab eo p e
+        ELSE Pexp_fun (labelled lab) eo p e END
     | pel ->
         if lab = "" && eo = None then Pexp_function pel
         else failwith "internal error: bad ast in ocaml_pexp_function"
@@ -766,10 +839,14 @@ value ocaml_psig_class_type =
 
 value ocaml_psig_exception loc s ed =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_exception (mkloc loc s) ed
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     Psig_exception
       {pext_name = mkloc loc s; pext_kind = Pext_decl ed None;
        pext_loc = loc; pext_attributes = []}
+  ELSE
+    Psig_exception
+      {pext_name = mkloc loc s; pext_kind = Pext_decl (Pcstr_tuple ed) None;
+       pext_loc = loc; pext_attributes = []}
   END
 ;
 
@@ -844,8 +921,10 @@ value ocaml_psig_type stl =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
     Psig_type stl
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     let stl = List.map (fun (s, t) -> t) stl in Psig_type stl
+  ELSE
+    let stl = List.map (fun (s, t) -> t) stl in Psig_type Recursive stl
   END
 ;
 
@@ -866,10 +945,14 @@ value ocaml_pstr_eval e =
 
 value ocaml_pstr_exception loc s ed =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_exception (mkloc loc s) ed
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     Pstr_exception
       {pext_name = mkloc loc s; pext_kind = Pext_decl ed None;
        pext_loc = loc; pext_attributes = []}
+  ELSE
+    Pstr_exception
+      {pext_name = mkloc loc s; pext_kind = Pext_decl (Pcstr_tuple ed) None;
+       pext_loc = loc; pext_attributes = []}
   END
 ;
 
@@ -962,8 +1045,10 @@ value ocaml_pstr_type stl =
   ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
     Pstr_type stl
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     let stl = List.map (fun (s, t) -> t) stl in Pstr_type stl
+  ELSE
+    let stl = List.map (fun (s, t) -> t) stl in Pstr_type Recursive stl
   END
 ;
 
@@ -1091,15 +1176,19 @@ value ocaml_pcf_valvirt =
 value ocaml_pcf_virt (s, pf, t, loc) =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_virt (s, pf, t, loc)
   ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pcf_virt (mkloc loc s, pf, t)
-  ELSE Pcf_val (mkloc loc s, Immutable, Cfk_virtual t) END
+  ELSE Pcf_method (mkloc loc s, pf, Cfk_virtual t) END
 ;
 
 value ocaml_pcl_apply =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSIFDEF OCAML_VERSION <= OCAML_2_04 THEN
     Some (fun ce lel -> Pcl_apply ce (List.map snd lel))
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     Some (fun ce lel -> Pcl_apply ce lel)
+  ELSE
+    Some
+      (fun ce lel ->
+         Pcl_apply ce (List.map (fun (l, e) -> (labelled l, e)) lel))
   END
 ;
 
@@ -1118,8 +1207,10 @@ value ocaml_pcl_fun =
     None
   ELSIFDEF OCAML_VERSION <= OCAML_2_04 THEN
     Some (fun lab ceo p ce -> Pcl_fun p ce)
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     Some (fun lab ceo p ce -> Pcl_fun lab ceo p ce)
+  ELSE
+    Some (fun lab ceo p ce -> Pcl_fun (labelled lab) ceo p ce)
   END
 ;
 
@@ -1180,8 +1271,10 @@ value ocaml_pcty_fun =
     Some (fun lab t ct -> Pcty_fun t ct)
   ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     Some (fun lab t ct -> Pcty_fun lab t ct)
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_03_0 THEN
     Some (fun lab t ct -> Pcty_arrow lab t ct)
+  ELSE
+    Some (fun lab t ct -> Pcty_arrow (labelled lab) t ct)
   END
 ;
 
@@ -1208,6 +1301,10 @@ value ocaml_pdir_bool =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN None
   ELSE Some (fun b -> Pdir_bool b) END
 ;
+value ocaml_pdir_int i s =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Pdir_int s
+  ELSE Pdir_int i None END
+;
 
 value ocaml_pwith_modsubst =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
@@ -1419,6 +1516,16 @@ ELSE
   value printf_ksprintf = Printf.ksprintf;
 END;
 
+value char_uppercase =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN Char.uppercase
+  ELSE Char.uppercase_ascii END
+;
+
+value string_capitalize =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN String.capitalize
+  ELSE String.capitalize_ascii END
+;
+
 value string_contains =
   IFDEF OCAML_VERSION <= OCAML_2_00 THEN
     fun s c ->
@@ -1443,11 +1550,26 @@ value string_create =
   ELSE Bytes.create END
 ;
 
+value string_lowercase =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN String.lowercase
+  ELSE String.lowercase_ascii END
+;
+
 value string_unsafe_set =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.unsafe_set
   ELSE Bytes.unsafe_set END
 ;
 
+value string_uncapitalize =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN String.uncapitalize
+  ELSE String.uncapitalize_ascii END
+;
+
+value string_uppercase =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN String.uppercase
+  ELSE String.uppercase_ascii END
+;
+
 value string_set =
   IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.set
   ELSE Bytes.set END
