@@ -7,7 +7,7 @@
 
 open Printf;
 
-value version = "6.14";
+value version = "6.16";
 value syntax_name = ref "";
 
 value gram =
@@ -532,5 +532,5 @@ add_option "-pmode"
       }))
   "Print the current mode and exit.";
 
-add_option "-dquot" (Arg.Set_string Quotation.default)
+add_option "-dquot" (Arg.String (fun s -> Quotation.default.val := s))
   "<name> Set default quotation.";
