@@ -23,7 +23,14 @@ let ocaml_location (fname, lnum, bolp, lnuml, bolpl, bp, ep) =
    Location.loc_ghost = bp = 0 && ep = 0}
 ;;
 
-(* *)
+let loc_none =
+  let loc =
+    {Lexing.pos_fname = "_none_"; Lexing.pos_lnum = 1; Lexing.pos_bol = 0;
+     Lexing.pos_cnum = -1}
+  in
+  {Location.loc_start = loc; Location.loc_end = loc;
+   Location.loc_ghost = true}
+;;
 
 let mkloc loc txt = txt;;
 let mknoloc txt = txt;;
@@ -55,6 +62,17 @@ let list_map_check f l =
   loop [] l
 ;;
 
+(* *)
+
+let mkopt t lab =
+  if lab = "" then t
+  else if lab.[0] = '?' then
+    {ptyp_desc =
+      Ptyp_constr (mknoloc (Ldot (Lident "*predef*", "option")), [t]);
+     ptyp_loc = loc_none}
+  else t
+;;
+
 let ocaml_value_description vn t p = {pval_type = t; pval_prim = p};;
 
 let ocaml_class_type_field loc ctfd = ctfd;;
@@ -104,7 +122,7 @@ let ocaml_ptype_record ltl priv = Ptype_record ltl;;
 
 let ocaml_ptype_variant ctl priv = Some (Ptype_variant ctl);;
 
-let ocaml_ptyp_arrow lab t1 t2 = Ptyp_arrow (lab, t1, t2);;
+let ocaml_ptyp_arrow lab t1 t2 = Ptyp_arrow (lab, mkopt t1 lab, t2);;
 
 let ocaml_ptyp_class li tl ll = Ptyp_class (mknoloc li, tl, ll);;
 
@@ -131,7 +149,21 @@ let ocaml_package_type li ltl =
   mknoloc li, List.map (fun (li, t) -> mkloc t.ptyp_loc li, t) ltl
 ;;
 
+let ocaml_pconst_char c = Const_char c;;
+let ocaml_pconst_int i = Const_int i;;
+let ocaml_pconst_float s = Const_float s;;
+
 let ocaml_const_string s = Const_string s;;
+let ocaml_pconst_string s so = Const_string s;;
+
+let pconst_of_const =
+  function
+    Const_int i -> ocaml_pconst_int i
+  | Const_char c -> ocaml_pconst_char c
+  | Const_string s -> ocaml_pconst_string s None
+  | Const_float s -> ocaml_pconst_float s
+  | _ -> failwith "pconstant of constant"
+;;
 
 let ocaml_const_int32 = Some (fun s -> Const_int32 (Int32.of_string s));;
 
@@ -392,6 +424,7 @@ let ocaml_pcty_fun = Some (fun lab t ct -> Pcty_fun (lab, t, ct));;
 let ocaml_pcty_signature = Some (fun (t, cil) -> Pcty_signature (t, cil));;
 
 let ocaml_pdir_bool = Some (fun b -> Pdir_bool b);;
+let ocaml_pdir_int i s = Pdir_int s;;
 
 let ocaml_pwith_modsubst =
   Some (fun loc me -> Pwith_modsubst (mkloc loc me))
@@ -477,14 +510,24 @@ let pervasives_set_binary_mode_out = Pervasives.set_binary_mode_out;;
 
 let printf_ksprintf = Printf.ksprintf;;
 
+let char_uppercase = Char.uppercase;;
+
+let string_capitalize = String.capitalize;;
+
 let string_contains = String.contains;;
 
 let string_copy = String.copy;;
 
 let string_create = String.create;;
 
+let string_lowercase = String.lowercase;;
+
 let string_unsafe_set = String.unsafe_set;;
 
+let string_uncapitalize = String.uncapitalize;;
+
+let string_uppercase = String.uppercase;;
+
 let string_set = String.set;;
 
 let array_create = Array.create;;
