@@ -7,7 +7,7 @@
 
 open Printf;;
 
-let version = "6.14";;
+let version = "6.16";;
 let syntax_name = ref "";;
 
 let gram =
@@ -473,5 +473,5 @@ add_option "-pmode"
         exit 0))
   "Print the current mode and exit.";;
 
-add_option "-dquot" (Arg.Set_string Quotation.default)
+add_option "-dquot" (Arg.String (fun s -> Quotation.default := s))
   "<name> Set default quotation.";;
